@@ -9,6 +9,33 @@ import os
 from sklearn.decomposition import PCA
 import numpy as np
 from tqdm import tqdm
+import lightning as pl
+from lightning.pytorch.callbacks import ModelCheckpoint
+
+
+
+def load_lightning_ckpt_into_model(model, ckpt_path, device):
+
+    raw = torch.load(ckpt_path, map_location=device)
+
+    # 1) 取出真正的 state_dict
+    sd = raw["state_dict"] if isinstance(raw, dict) and "state_dict" in raw else raw
+
+    # 2) 清理常见前缀
+    cleaned = {}
+    for k, v in sd.items():
+        kk = k
+        for prefix in ("model.", "module.", "net."):
+            if kk.startswith(prefix):
+                kk = kk[len(prefix):]
+        cleaned[kk] = v
+
+    # 3) 加载（先 strict=False 看缺失/多余键）
+    missing, unexpected = model.load_state_dict(cleaned, strict=False)
+    print("[load_state_dict] missing keys:", missing)
+    print("[load_state_dict] unexpected keys:", unexpected)
+    return model
+
 
 def noise_map_exp(model, data, num_exp=10, noise_level=0.1):
     exp_feature_num = int(data.shape[1] // num_exp)
@@ -73,6 +100,8 @@ class DMTLearn:
                  loss_type='L',
                  all_g_l_weight=0.5,
                  dim_high=500,  # Default value, can be adjusted as needed
+                 save_ckpt_path=None,
+                 load_ckpt_path=None,
                  ):
         
         seed_everything(random_state)
@@ -99,6 +128,32 @@ class DMTLearn:
         self.loss_type = loss_type
         self.all_g_l_weight = all_g_l_weight
         self.dim_high = dim_high  # Default value, can be adjusted as needed
+        self.save_ckpt_path = save_ckpt_path
+        self.enable_checkpointing = False
+        self.load_ckpt_path = load_ckpt_path
+        
+        
+        callbacks = []
+        
+        if self.save_ckpt_path is not None:
+            # Ensure the directory exists
+            # self.save_ckpt_path = 'ckpt/dmt.ckpt'
+            
+            path_dir = self.save_ckpt_path.rsplit('/', 1)[0]
+            if not os.path.exists(path_dir):
+                os.makedirs(path_dir, exist_ok=True)
+            
+            lightning_save_ckpt_callback = ModelCheckpoint(
+                dirpath=path_dir,
+                filename=os.path.basename(self.save_ckpt_path),
+                save_top_k=1,
+                monitor='epoch',
+                mode='max',
+                save_weights_only=True,
+            )
+
+            callbacks.append(lightning_save_ckpt_callback)
+            self.enable_checkpointing = True
 
         if num_devices > 1:
             self.trainer = Trainer(
@@ -106,20 +161,23 @@ class DMTLearn:
                 devices=num_devices,
                 accelerator='gpu',
                 strategy='ddp',
-                enable_checkpointing=False,
+                enable_checkpointing=self.enable_checkpointing,
                 logger=False,
+                callbacks=callbacks,
             )
         else:
             self.trainer = Trainer(
                 max_epochs=max_epochs,
                 devices=num_devices,
                 accelerator='gpu',
-                enable_checkpointing=False,
+                enable_checkpointing=self.enable_checkpointing,
                 logger=False,
+                callbacks=callbacks,
             )
         
     def transform(self, data, batch_size=1000):
         data_tensor = torch.tensor(data).float()
+        
         # import pdb; pdb.set_trace()
         
         num_loop = data_tensor.size(0) // batch_size
@@ -159,7 +217,8 @@ class DMTLearn:
 
 
     def fit(self, dm):
-        self.trainer.fit(self.model, dm) 
+        
+        self.trainer.fit(self.model, dm)
 
 
     def extract_embeddings_from_loader(self):
@@ -214,6 +273,65 @@ class DMTLearn:
         return all_datas, all_embeddings, all_exp_noist_test_result_dict, all_xmask, all_labels
 
 
+    def ckpt_transform(self, data, ckpt, neighbors_index=None, return_high_dim=False):
+
+        print('start init model')
+        self.model = DMTEVT_model(
+            num_input_dim=data.shape[1],
+            lr=self.lr,
+            nu_lat=self.nu,
+            nu_emb=self.nu,
+            exaggeration_lat=self.exaggeration,
+            exaggeration_emb=self.exaggeration,
+            sample_rate_feature=self.sample_rate_feature,
+            T_hidden_size=self.T_hidden_size,
+            num_use_moe=self.num_use_moe,
+            T_num_layers=self.T_num_layers,
+            max_epochs=self.max_epochs*5,
+            weight_decay=0.000000001,
+            use_orthogonal=self.use_orthogonal,
+            use_high_manifold_loss=self.use_high_manifold_loss,
+            norm_loss=self.norm_loss,
+            temp=self.temp,
+            loss_type=self.loss_type,
+            all_g_l_weight=self.all_g_l_weight,
+            vis_dim=self.n_components,
+            dim_high=self.dim_high
+        )
+        
+
+        self.model = self.model.to(self.device)
+
+        # if neighbors_index is None:
+        #     neighbors_index = Preprocessing(data, self.n_neighbors, 64)
+        
+        # print('start init MyDataModule')
+        # dm = MyDataModule(
+        #     data_npy=data,
+        #     data_name='DMTLEARN',
+        #     pca_dim=64,
+        #     batch_size=self.batch_size,
+        #     num_workers=self.n_jobs,
+        #     K=self.n_neighbors,
+        #     n_f_per_cluster=3,
+        #     l_token=10,
+        #     data_path='./data_path',
+        #     neighbors_index=neighbors_index,
+        # )
+        # self.dm = dm
+        
+        # print('start fit')
+        # self.fit(dm)
+        self.model = load_lightning_ckpt_into_model(self.model, ckpt, self.device)
+        print('start transform')
+        vis_results, high_dim_results = self.transform(data)
+
+        if return_high_dim:
+            return vis_results, high_dim_results
+        else:
+            return vis_results
+
+
     def fit_transform(self, data, neighbors_index=None, return_high_dim=False):
 
         print('start init model')
@@ -239,6 +357,7 @@ class DMTLearn:
             vis_dim=self.n_components,
             dim_high=self.dim_high
         )
+        
 
         self.model = self.model.to(self.device)
 
